package dk.sosi.sts.integrations.blurring;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.CareProvider;
import dk.sosi.seal.model.SystemIDCard;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;

import java.time.Instant;
import java.util.Date;
import java.util.concurrent.atomic.AtomicReference;

public class IdCardFactory {
    private final SOSIFactory sosiFactory;
    private String itSystemName, careProviderId, careProviderOrg;

    private final AtomicReference<SystemIDCard> sharedSystemIdCard = new AtomicReference<>();

    // injected by spring
    public void setItSystemName(String itSystemName) { this.itSystemName = itSystemName; }
    public void setCareProviderId(String careProviderId) { this.careProviderId = careProviderId; }
    public void setCareProviderOrg(String careProviderOrg) { this.careProviderOrg = careProviderOrg; }

    public IdCardFactory(SOSIFactory sosiFactory) {
        this.sosiFactory = sosiFactory;
    }

    public SystemIDCard getIdCard() {
        CareProvider careProvider = new CareProvider(
                SubjectIdentifierTypeValues.CVR_NUMBER, careProviderId, careProviderOrg);
        AuthenticationLevel authenticationLevel = AuthenticationLevel.VOCES_TRUSTED_SYSTEM;

        if (sharedSystemIdCard.get() == null || renewalNeeded()) {
            sharedSystemIdCard.set(sosiFactory.createNewSystemIDCard(itSystemName, careProvider, authenticationLevel, null, null, null, null));
        }
        return sharedSystemIdCard.get();
    }

    private boolean renewalNeeded() {
        return Date.from(Instant.now().plusSeconds(300)).after(sharedSystemIdCard.get().getExpiryDate());
    }
}